## Before running the function f.mfb, load the library  MASS using:
## library(MASS)
##
## In R, also load the library nlme, using:
## library(nlme)
##
## In S-PLUS, it may be necessary to load the library nlme3, using:
## library(nlme3)

############### f.mfb function #######################################

"f.mfb" <- function(file = "mfbraw.txt"){
##
## THE AE MODEL APPLIED TO CONTINUOUS BIRTH WEIGHT DATA
##

## read data

mfb <- read.table(file, header = T)


## reshape to long. R users could use "reshape" function


member <- rep(1:3, each = nrow(mfb))
fam <- rep(1:nrow(mfb), 3)
bwt <- c(mfb$bwt1, mfb$bwt2, mfb$bwt3)
male <- c(mfb$male1, mfb$male2, mfb$male3)
mothage <- c(mfb$mothage1, mfb$mothage2, mfb$mothage3)
birthyr <- c(mfb$byr1, mfb$byr2, mfb$byr3)
parity <- c(mfb$parity1, mfb$parity2, mfb$parity3)


## transform variables

first <- as.numeric(parity == 0)
birthyr <- birthyr - 67
midage <- as.numeric(mothage >= 20 & mothage <= 35)
highage <- as.numeric(mothage > 35)

MM <- as.numeric(member == 1)
FF <- as.numeric(member == 2)
CC <- as.numeric(member == 3)

var1 <- MM + CC/2
var2 <- FF + CC/2
var3 <- CC/sqrt(2)

## estimate model as described in Section 7.1


mfblong <- data.frame(bwt = bwt, male = male, first = first, midage = midage,
  highage = highage, birthyr = birthyr, var1 = var1, var2 = var2, 
  var3 = var3, fam = fam)

res <- lme(bwt ~ male + first + midage + highage + birthyr, 
   random = list(fam = pdIdent(~var1 + var2 + var3 -1)), data = mfblong, method = "ML")
print(summary(res))

#### get heritability and confidence interval
lns2 <- attr(res$apVar, "Pars")[1]
lns1 <- attr(res$apVar, "Pars")[2]
var22 <- res$apVar[1, 1]
cov21 <- res$apVar[2, 1]
var11 <- res$apVar[2, 2]

herit <- exp(2*lns2)/(exp(2*lns1) + exp(2*lns2))

cat("\n")
cat("Heritability:", herit, "\n")

## confidence interval for heritability
## using standard error of heritability
## (delta method used once)

deriv1 <- -2*exp(2*lns1)*exp(2*lns2)/(exp(2*lns1) + exp(2*lns2))^2
deriv2 <- -2*exp(4*lns2)/(exp(2*lns1) + exp(2*lns2))^2 + 2*exp(2*lns2)/(exp(2*lns1) + exp(2*lns2))
se <- sqrt(var11*deriv1^2 + var22*deriv2^2 + 2*cov21*deriv1*deriv2)
cat("95% CI (no transformation):     ", herit - qnorm(.975)*se, "to", herit + qnorm(.975)*se, "\n")

## confidence interval for heritability
## using probit transformation and appropriate standard error 
## (delta method used twice)

se.qnorm <- se/dnorm(qnorm(herit))
pnorm(qnorm(herit) - qnorm(.975)*se.qnorm)
pnorm(qnorm(herit) + qnorm(.975)*se.qnorm)
cat("95% CI (probit transformation): ", pnorm(qnorm(herit) - qnorm(.975)*se.qnorm), "to", 
  pnorm(qnorm(herit) + qnorm(.975)*se.qnorm), "\n")


## simplified version when there is no common environment
diff <- lns1 - lns2
se.diff <- sqrt(var11 + var22 - 2*cov21)
diff.lower <- diff - 1.96*se.diff
diff.upper <- diff + 1.96*se.diff
herit.upper <- 1/(exp(2*diff.lower) + 1)
herit.lower <- 1/(exp(2*diff.upper) + 1)
cat("95% CI (simpler transformation):", herit.lower, "to", herit.upper, "\n")

## simulated confidence intervals
set.seed(24)
samp <- exp(2*mvrnorm(n = 50000, mu = c(lns1, lns2), Sigma = matrix(c(var11, cov21, cov21, var22), ncol = 2), empirical = T))
samp.herit <- samp[,2]/(samp[,1] + samp[,2])
CI <- quantile(samp.herit, c(0.025, 0.975))
cat("95% CI (simulated):             ", CI[1], "to", CI[2], "\n")

return(invisible(res))
}
