
**
** MODELS FOR DICHOTOMOUS DEPRESSION DATA
**

** This do-file uses gllamm, a user-contributed program that
** can be downloaded from within Stata using the command:
**
** ssc install gllamm, replace
**
** (see http://www.gllamm.org for documentation, etc.)


version 9.2
set more off

** prepare data
use twindep, clear
set matsize 60

tab twin2 twin1 [fweight=num3] if dzmz=="mz"
tab twin2 twin1 [fweight=num3] if dzmz=="dz"
gen num = cond(dzmz=="mz",590,440)

gen pair=_n
gen mz=dzmz=="mz"
gen dz=dzmz=="dz"

reshape long twin, i(pair) j(id)

egen cases=sum(twin), by(pair)
tab cases [fweight=num3] if dzmz=="mz" & id==1
tab cases [fweight=num3] if dzmz=="dz" & id==1


/**************** ACE or ADE model, parameterization 2 *****************/

** create level-2 identifier "k"
gen clus = id if dz==1
replace clus = pair if mz==1

** estimate three-level random intercept probit model
gllamm twin, i(clus pair) weight(num) family(binom) link(probit) ///
   adapt nip(12)
   
** compute implied expected counts
gllapred ll, ll
gen ace2 = num*exp(ll)
tab cases [iweight=ace2] if dzmz=="mz" & id==1
tab cases [iweight=ace2] if dzmz=="dz" & id==1
drop ll


** calculate model-implied variance components
* ACE
nlcom (var_A: 2*[clus1]_cons^2) (var_C: [pair2]_cons^2- [clus1]_cons^2)

* ADE
nlcom (var_A: 3*[pair2]_cons^2 - [clus1]_cons^2) ///
      (var_D: 2*([clus1]_cons^2 - [pair2]_cons^2))



/**********AE model, parameterization 1 *************************************/

** create variables multiplying the random coefficients a^(2) and a^(3)
gen cov1 = sqrt(.5)*dz
gen cov2 = mz+sqrt(.5)*dz

** estimate 3-level random coefficient probit model
eq cov1: cov1
eq cov2: cov2

constraint def 1 [id1_1]cov1 = [pai2_1]cov2 /* constrain variances */

gllamm twin, i(id pair) nrf(1 1) eqs(cov1 cov2) weight(num) constr(1) ///
  adapt nocor nip(8) f(binom) l(probit) 

**calculate model-implied expected counts
gllapred ll, ll

gen ae=num*exp(ll)
tab cases [iweight=ae] if dzmz=="mz" & id==1
tab cases [iweight=ae] if dzmz=="dz" & id==1
drop ll

**define program for 95% confidence interval for heritability
capture program drop confint
program define confint, eclass
    _coef_table, first plus
    estimates store model1
    * rename column name for [id1_1]cov1 to _cons
    matrix b=e(b)
    matrix b[1,1]=0
    matrix colnames b = _cons _cons _cons
    matrix V=e(V)
    matrix colnames V = _cons _cons _cons
    matrix rownames V = _cons _cons _cons
    ereturn post b V
    noi _diparm id1_1, label(heritability) func(@^2/(@^2+1)) ///
    deriv( -2*@^3/(@^2+1)^2+2*@/((@^2+1))) ////
    level(95) ci(probit)
    disp in smcl in gr "{hline 13}" "{c BT}" "{hline 64}" _n
    qui estimates restore model1
end

**run program to get confidence interval
confint


/********** ACE model, parameterization 1 **********************************/

**estimate same as for AE, but include a random intercept at level 3
gen cons=1
eq cons: cons
eq cov1: cov1
eq cov2: cov2

constraint def 1 [id1_1]cov1 = [pai2_1]cov2
gllamm twin, i(id pair) nrf(1 2) eqs(cov1 cov2 cons) weight(num) constr(1) ///
  family(binom) link(probit) adapt nocor nip(8) 
  
**model-implied expected counts
gllapred ll, ll 
gen ace1=num*exp(ll)
tab cases [iweight=ace1] if dzmz=="mz" & id==1
tab cases [iweight=ace1] if dzmz=="dz" & id==1
drop cons ll

  
/********** ADE model, parameterization 1 **************/

**create variables multiplying the random coefficients
gen cov3 = sqrt(3/4)*dz
gen cov4 = mz+sqrt(1/4)*dz

**estimate probit model with four random effects
eq a1: cov1
eq a2: cov2
eq d1: cov3
eq d2: cov4

**constrain variances
constraint def 1 [id1_1]cov1 = [pai2_1]cov2
constraint def 2 [id1_2]cov3 = [pai2_2]cov4

set matsize 60
gllamm twin, i(id pair) nrf(2 2) eqs(a1 d1 a2 d2) weight(num) constr(1 2) ///
  family(binom) link(probit) adapt nocor nip(8) 

**calculate model-implied expected counts
gllapred ll, ll
gen ade1=num*exp(ll)
tab cases [iweight=ade1] if dzmz=="mz" & id==1
tab cases [iweight=ade1] if dzmz=="dz" & id==1
drop ll
